'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var doTextMatchPattern = exports.doTextMatchPattern = function doTextMatchPattern(text, pattern) {
  var containsPattern = pattern.includes('*') || pattern.includes('#');

  if (!containsPattern) {
    return text === pattern;
  }

  var regExp = pattern.replace(/\./g, '\\.');
  regExp = regExp.replace(/\*/g, '\\w+');
  regExp = regExp.replace(/#/g, '.+?');
  regExp = '^' + regExp + '$';

  regExp = new RegExp(regExp);

  return regExp.test(text);
};

exports.default = function () {
  return {
    subscribers: {},
    queued: {},
    subscribe: function subscribe() {
      var _this = this;

      // if we don't provide 3 arguments then we get 1st argument as topic and 2nd
      //  as callback, channel is 'default'

      var channel = arguments[0];
      var topicPattern = arguments[1];
      var callback = arguments[2];

      if (arguments.length === 2) {
        channel = 'default';
        topicPattern = arguments[0];
        callback = arguments[1];
      }

      if (this.subscribers[channel] === undefined) {
        this.subscribers[channel] = {};
      }

      if (this.subscribers[channel][topicPattern] === undefined) {
        this.subscribers[channel][topicPattern] = { subscribers: [] };
      }

      this.subscribers[channel][topicPattern].subscribers.push(callback);

      if (this.queued[channel] !== undefined) {
        var matchingQueuedTopics = Object.keys(this.queued[channel]).filter(function (t) {
          return doTextMatchPattern(t, topicPattern);
        });

        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = matchingQueuedTopics[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var topic = _step.value;
            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
              for (var _iterator2 = this.queued[channel][topic][Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                var data = _step2.value;

                callback(data.data, {
                  channel: data.channel,
                  topic: data.topic
                });
              }
            } catch (err) {
              _didIteratorError2 = true;
              _iteratorError2 = err;
            } finally {
              try {
                if (!_iteratorNormalCompletion2 && _iterator2.return) {
                  _iterator2.return();
                }
              } finally {
                if (_didIteratorError2) {
                  throw _iteratorError2;
                }
              }
            }

            this.queued[channel][topic] = [];
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }
      }

      return {
        unsubscribe: function unsubscribe() {
          var index = _this.subscribers[channel][topicPattern].subscribers.findIndex(function (s) {
            return s === callback;
          });

          if (index !== -1) {
            _this.subscribers[channel][topicPattern].subscribers.splice(index, 1);
          }
        }
      };
    },
    publish: function publish() {
      var _this2 = this;

      var channel = arguments[0];
      var topic = arguments[1];
      var data = arguments[2];
      var onlyOnce = arguments[3];

      if (arguments.length === 3) {
        channel = arguments[0];
        topic = arguments[1];
        data = arguments[2];
        onlyOnce = false;
      } else if (arguments.length === 2) {
        channel = 'default';
        topic = arguments[0];
        data = arguments[1];
        onlyOnce = false;
      }

      if (this.subscribers[channel] === undefined) {
        if (this.queued[channel] === undefined) {
          this.queued[channel] = {};
        }
        if (this.queued[channel][topic] === undefined) {
          this.queued[channel][topic] = [];
        }

        this.queued[channel][topic].push({
          channel: channel,
          topic: topic,
          data: data
        });

        return;
      }

      var matchedTopicPatterns = Object.keys(this.subscribers[channel]).filter(function (t) {
        return doTextMatchPattern(topic, t);
      });

      var matchingSubscribersCount = matchedTopicPatterns.map(function (t) {
        return _this2.subscribers[channel][t].subscribers.length;
      }).reduce(function (acc, len) {
        return acc += len;
      });

      if (matchingSubscribersCount === 0) {
        if (this.queued[channel] === undefined) {
          this.queued[channel] = {};
        }
        if (this.queued[channel][topic] === undefined) {
          this.queued[channel][topic] = [];
        }

        this.queued[channel][topic].push({
          channel: channel,
          topic: topic,
          data: data
        });

        return;
      }

      var _iteratorNormalCompletion3 = true;
      var _didIteratorError3 = false;
      var _iteratorError3 = undefined;

      try {
        for (var _iterator3 = matchedTopicPatterns.reverse()[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
          var topicPattern = _step3.value;

          if (onlyOnce) {
            var subCount = this.subscribers[channel][topicPattern].subscribers.length;
            this.subscribers[channel][topicPattern].subscribers[subCount - 1](data, {
              channel: channel,
              topic: topic
            });
            return;
          } else {
            this.subscribers[channel][topicPattern].subscribers.forEach(function (s) {
              return s(data, {
                channel: channel,
                topic: topic
              });
            });
          }
        }
      } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion3 && _iterator3.return) {
            _iterator3.return();
          }
        } finally {
          if (_didIteratorError3) {
            throw _iteratorError3;
          }
        }
      }
    }
  };
};
